@@ -1,17 +1,16 @@
-#'@title [DBR-lambda] procedure
+#'@title [HBR-\eqn{\lambda}] procedure
 #'
 #'@description
-#'Apply the [DBR-lambda] procedure,
+#'Apply the [HBR-\eqn{\lambda}] procedure,
 #'with or without computing the critical constants,
 #'to a set of p-values and their discrete support.
 #'
 #'@details
-#'
 #'[DBR-lambda] is the discrete version of the [Blanchard-Roquain-lambda] procedure (see References),
 #'the authors of the latter suggest to take \code{lambda = alpha} (see their Proposition 17),
 #'which explains the choice of the default value here. 
 #'
-#'This version: 2018-11-13.
+#'This version: 2019-06-18.
 #'
 #'@section References:
 #'G. Blanchard and E. Roquain (2009). Adaptive false discovery rate control under independence and dependence. Journal of Machine Learning Research, 10, 2837-2871.
@@ -19,25 +18,26 @@
 #'@seealso
 #'\code{\link{discrete.BH}}, \code{\link{DiscreteFDR}}
 #'
-#'@templateVar stepf FALSE
-#'@templateVar pv.numer FALSE
-#'@templateVar pv.denom FALSE
+#'@templateVar pvalues FALSE
+#'@templateVar stepUp FALSE
 #'@templateVar alpha TRUE
-#'@templateVar sorted.pv FALSE
-#'@templateVar pCDFlist TRUE
+#'@templateVar sorted_pv FALSE
+#'@templateVar support FALSE
 #'@templateVar raw.pvalues TRUE
+#'@templateVar pCDFlist TRUE
 #'@templateVar direction FALSE
 #'@templateVar ret.crit.consts TRUE
-#'@templateVar sorted.num FALSE
-#'@templateVar t FALSE
 #'@templateVar lambda TRUE
+#'@templateVar adaptive FALSE
 #'@template param 
 #'
 #'@template example
 #'@examples
 #'
-#'DBR.su.fast <- DBR(raw.pvalues, pCDFlist)
-#'DBR.su.crit <- DBR(raw.pvalues, pCDFlist, ret.crit.consts = TRUE)
+#'DBR.fast <- DBR(raw.pvalues, pCDFlist)
+#'summary(DBR.fast)
+#'DBR.crit <- DBR(raw.pvalues, pCDFlist, ret.crit.consts = TRUE)
+#'summary(DBR.crit)
 #'
 #'@templateVar DBR TRUE
 #'@template return
@@ -61,34 +61,24 @@ DBR <- function(raw.pvalues, pCDFlist, alpha = 0.05, lambda = NULL, ret.crit.con
   o <- order(pvec)                                                                                       
   sorted.pvals <- pvec[o]
   #--------------------------------------------
-  #       define step functions corresponding to distribution functions of p-values under Null-Hypotheses
+  #       construct the vector of all values of all supports of the p-values
   #--------------------------------------------
-  stepf <- build.stepfuns(pCDFlist)
-  #--------------------------------------------
-  pv.list.all <- unique(sort(as.numeric(unlist(pCDFlist))))
+  pv.list.all <- sort(unique(as.numeric(unlist(pCDFlist))))
   #--------------------------------------------
   #        Compute [DBR-lambda] significant p-values,
   #        their indices and the number of rejections
   #--------------------------------------------
   if(ret.crit.consts){
-    # construct the vector of all values of all supports of the p-values
-    pv.list.all <- sort(unique(as.numeric(unlist(pCDFlist))))
-    # apply the shortcut drawn from Corollary 3, that is
-    # c.1 >= the effective critical value associated to min((1 - lambda) * alpha/m , lambda)
-    pv.list <- short.eff(pv.list.all, min((1 - lambda) * alpha/m , lambda))
-    # then re-add the observed p-values (needed to compute the adjusted p-values),
-    # because we may have removed some of them the shortcut
-    pv.list <- sort(unique(c(pv.list, sorted.pvals)))
     # compute transformed support
-    y <- kernel.DBR.crit(stepf, pv.list, lambda, alpha, sorted.pvals)
+    y <- kernel_DBR_crit(pCDFlist, pv.list.all, sorted.pvals, lambda, alpha)
     # find critical constants
     crit.constants <- y$crit.consts
     idx <- which(sorted.pvals <= crit.constants)   
   }
   else{
     # compute transformed p-values
-    y <- kernel.DBR.fast(stepf, sorted.pvals, lambda)
-    idx <- which( y <= alpha)
+    y <- kernel_DBR_fast(pCDFlist, sorted.pvals, lambda)
+    idx <- which(y <= alpha)
   }
   m.rej <- length(idx)
   if(m.rej){
@@ -101,13 +91,14 @@ DBR <- function(raw.pvalues, pCDFlist, alpha = 0.05, lambda = NULL, ret.crit.con
   #--------------------------------------------
   #       Create output list
   #--------------------------------------------
-  output <- list(Rejected = pvec.rej, Indices = idx, Alpha = m.rej * alpha / m, k.hat = m.rej, Lambda = lambda)
+  output <- list(Rejected = pvec.rej, Indices = idx, Alpha = m.rej * alpha / m, Num.rejected = m.rej, Lambda = lambda)
   if(ret.crit.consts){
-    output <- c(output, list(Critical.constants = crit.constants))
+    # add critical values to output list
+    output$Critical.values = crit.constants
     # compute adjusted p-values
     # recall that transformed p-values where max_i F_i(p) > lambda,
     # that is for indices > y$m.lambda, are set to 1
-    pv.adj <- rev(cummin(rev(pmin(y$pval.transf / c(seq_len(y$m.lambda), rep(1, m - y$m.lambda)), 1))))
+    pv.adj <- rev(cummin(rev(pmin(y$pval.transf, 1)))) # / c(seq_len(y$m.lambda), rep(1, m - y$m.lambda))
   }
   else{
     # compute adjusted p-values
@@ -115,6 +106,22 @@ DBR <- function(raw.pvalues, pCDFlist, alpha = 0.05, lambda = NULL, ret.crit.con
   }
   # add adjusted p-values to output list
   ro <- order(o)
-  output <- c(output, list(Adjusted = pv.adj[ro]))
+  output$Adjusted = pv.adj[ro]
+  
+  # include details of the used algorithm as strings
+  output$Method <- paste("Discrete Blanchard-Roquain procedure (lambda = ", lambda, ")", sep = "")
+  output$Signif.level <- alpha
+  output$Tuning <- lambda
+  
+  # original test data (often included, e.g. when using 'binom.test()')
+  output$Data <- list()
+  output$Data$raw.pvalues <- raw.pvalues
+  output$Data$pCDFlist <- pCDFlist
+  # object names of the data as strings
+  output$Data$data.name <- paste(deparse(substitute(raw.pvalues)), "and", deparse(substitute(pCDFlist)))
+  
+  class(output) <- "DiscreteFDR"
+  return(output)
+  
   return(output)
 }
\ No newline at end of file
